/*
Copyright (c) 2000-2005 Joachim Henke

For conditions of distribution and use, see copyright notice in basE91.java
*/

import java.io.*;

public class b91enc
{
	private static void encode(InputStream is, OutputStream os)
	{
		int s;
		byte[] ibuf = new byte[53248];
		byte[] obuf = new byte[65536];
		basE91 b91 = new basE91();

		try {
			while ((s = is.read(ibuf)) > 0) {
				s = b91.encode(ibuf, s, obuf);
				os.write(obuf, 0, s);
			}
			s = b91.encEnd(obuf);
			os.write(obuf, 0, s);
		} catch (Exception e) {
			System.err.println(e);
		}
	}

	private static void encodeBr(InputStream is, OutputStream os)
	{
		int i, s;
		int n = 0;
		byte[] ibuf = new byte[53248];
		byte[] obuf = new byte[65536];
		char[] line = new char[76];
		PrintStream ps = new PrintStream(os);
		basE91 b91 = new basE91();

		try {
			while ((s = is.read(ibuf)) > 0) {
				s = b91.encode(ibuf, s, obuf);
				for (i = 0; i < s; ++i) {
					line[n++] = (char) obuf[i];
					if (n == 76) {
						ps.println(line);
						n = 0;
					}
				}
			}
		} catch (Exception e) {
			System.err.println(e);
		}
		s = b91.encEnd(obuf);
		for (i = 0; i < s; ++i) {
			line[n++] = (char) obuf[i];
			if (n == 76) {
				ps.println(line);
				n = 0;
			}
		}
		if (n > 0)
			ps.println(new String(line, 0, n));
	}

	private static void decode(InputStream is, OutputStream os)
	{
		int s;
		byte[] ibuf = new byte[65536];
		byte[] obuf = new byte[57344];
		basE91 b91 = new basE91();

		try {
			while ((s = is.read(ibuf)) > 0) {
				s = b91.decode(ibuf, s, obuf);
				os.write(obuf, 0, s);
			}
			s = b91.decEnd(obuf);
			os.write(obuf, 0, s);
		} catch (Exception e) {
			System.err.println(e);
		}
	}

	private static void errExit(String msg)
	{
		System.err.println("syntax error - " + msg + "\nTry `-h' option for more information.");
		System.exit(3);
	}

	public static void main(String[] args)
	{
		int i;
		boolean enc = true;
		boolean lbr = false;
		String ifn = null;
		String ofn = null;

		for (i = 0; i < args.length; ++i)
			if (args[i].length() == 2 && args[i].charAt(0) == '-')
				switch (args[i].charAt(1)) {
				case 'b':
					lbr = true;
					break;
				case 'd':
					enc = false;
					break;
				case 'h':
					System.out.println("Usage: b91enc [OPTION] infile [outfile]\n\n  -b\tbreak encoded output to lines of 76 characters (MIME compatible)\n  -d\tdecode a basE91 encoded file\n  -h\tdisplay this help and exit\n  -V\toutput version information and exit");
					return;
				case 'V':
					System.out.println("b91enc 0.5.1\nCopyright (c) 2000-2005 Joachim Henke");
					return;
				default:
					errExit("invalid option: " + args[i]);
				}
			else if (ifn == null)
				ifn = args[i];
			else if (ofn == null)
				ofn = args[i];
			else
				errExit("too many arguments: " + args[i]);
		if (ifn == null)
			errExit("file name missing");
		if (ofn == null)
			if (enc)
				ofn = ifn + (lbr ? "_b91.txt" : ".b91");
			else {
				String lifn = ifn.toLowerCase();
				if (ifn.length() > 4 && lifn.endsWith(".b91"))
					ofn = ifn.substring(0, ifn.length() - 4);
				else if (ifn.length() > 8 && lifn.endsWith("_b91.txt"))
					ofn = ifn.substring(0, ifn.length() - 8);
				else
					ofn = ifn + ".bin";
			}

		try {
			FileInputStream ifs = new FileInputStream(ifn);
			FileOutputStream ofs = new FileOutputStream(ofn);

			if (enc)
				if (lbr)
					encodeBr(ifs, ofs);
				else
					encode(ifs, ofs);
			else
				decode(ifs, ofs);
			ifs.close();
			ofs.close();
		} catch (Exception e) {
			System.err.println(e);
		}
	}
}
