/*
Copyright (c) 2000-2005 Joachim Henke

This software is provided 'as-is', without any express or implied warranty. In
no event will the authors be held liable for any damages arising from the use
of this software.

Permission is granted to anyone to use this software for any purpose, including
commercial applications, and to alter it and redistribute it freely, subject to
the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim
   that you wrote the original software. If you use this software in a product,
   an acknowledgment in the product documentation would be appreciated but is
   not required.

2. Altered source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/

public class basE91
{
	private int ebq, en, dbq, dn, dv;
	public final byte[] enctab;
	private final byte[] dectab;

	public int encode(byte[] ib, int n, byte[] ob)
	{
		int i, v;
		int o = 0;

		for (i = 0; i < n; ++i) {
			ebq |= (ib[i] & 255) << en;
			en += 8;
			if (en > 13) {
				v = ebq & 8191;
				if (v > 88) {
					ebq >>= 13;
					en -= 13;
				} else {
					v = ebq & 16383;
					ebq >>= 14;
					en -= 14;
				}
				ob[o++] = enctab[v % 91];
				ob[o++] = enctab[v / 91];
			}
		}
		return o;
	}

	public int encEnd(byte[] ob)
	{
		int o = 0;

		if (en > 0) {
			ob[o++] = enctab[ebq % 91];
			if (en > 7 || ebq > 90)
				ob[o++] = enctab[ebq / 91];
		}
		encReset();
		return o;
	}

	public void encReset()
	{
		ebq = 0;
		en = 0;
	}

	public int decode(byte[] ib, int n, byte[] ob)
	{
		int i, o = 0;

		for (i = 0; i < n; ++i) {
			if (dectab[ib[i]] < 0)
				continue;
			if (dv < 0)
				dv = dectab[ib[i]];
			else {
				dv += dectab[ib[i]] * 91;
				dbq |= dv << dn;
				dn += (dv & 8191) > 88 ? 13 : 14;
				do {
					ob[o++] = (byte) dbq;
					dbq >>= 8;
					dn -= 8;
				} while (dn > 7);
				dv = -1;
			}
		}
		return o;
	}

	public int decEnd(byte[] ob)
	{
		int o = 0;

		if (dv >= 0)
			ob[o++] = (byte) (dbq | dv << dn);
		decReset();
		return o;
	}

	public void decReset()
	{
		dbq = 0;
		dn = 0;
		dv = -1;
	}

	public basE91()
	{
		int i;
		String ts = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!#$%&()*+,./:;<=>?@[]^_`{|}~\"";

		enctab = ts.getBytes();
		dectab = new byte[256];
		for (i = 0; i < 256; ++i)
			dectab[i] = -1;
		for (i = 0; i < 91; ++i)
			dectab[enctab[i]] = (byte) i;
		encReset();
		decReset();
	}
}
