/*
Copyright (c) 2000-2005 Joachim Henke

For conditions of distribution and use, see copyright notice in base91.h
*/

#include <fcntl.h>
#include <libgen.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include "b91enc.h"
#include "base91.h"

unsigned char *ibf, *obf;
int ibs;
int ifd = STDIN_FILENO;
int ofd = STDOUT_FILENO;

float encode(void)
{
	int s, i = 0, o = 0;

	while ((s = read(ifd, ibf, ibs)) > 0) {
		i += s;
		s = b91_encode(ibf, s, obf);
		o += s;
		write(ofd, obf, s);
	}
	s = b91_enc_end(obf);	/* empty bit queue */
	o += s;
	write(ofd, obf, s);

	return i ? (float) o / i : 1;
}

void decode(void)
{
	int s;

	while ((s = read(ifd, ibf, ibs)) > 0) {
		s = b91_decode(ibf, s, obf);
		write(ofd, obf, s);
	}
	s = b91_dec_end(obf);	/* empty bit queue */
	write(ofd, obf, s);
}

int main(int argc, char **argv)
{
	int obs;
	char *ifn, *ofn, flg, opt;

	obs = -1;
	ifn = "from standard input";
	ofn = NULL;
	flg = strcmp(basename(argv[0]), "b91enc") ? FLG_D : 0;

	while ((opt = getopt(argc, argv, "dem:o:vhV")) != -1)
		switch (opt) {
		case 'd':
			flg |= FLG_D;
			break;
		case 'e':
			flg &= ~FLG_D;
			break;
		case 'm':
			if (sscanf(optarg, "%u%c", &obs, &opt) > 1)
				switch (opt | 32) {
				case 'b':
					break;
				case 'k':
					obs <<= 10;
					break;
				case 'm':
					obs <<= 20;
					break;
				default:
					fprintf(stderr, "Invalid SIZE suffix: %c\n", opt);
					return 1;
				}
			if (obs < 0) {
				fputs("Improperly formatted SIZE argument.\n", stderr);
				return 1;
			}
			break;
		case 'o':
			if (strcmp(optarg, "-"))
				ofn = optarg;
			break;
		case 'v':
			flg |= (flg & FLG_V) ? FLG_VV : FLG_V;
			break;
		case 'h':
			puts("Usage: b91enc [OPTION]... [FILE]\n"
				"With no FILE, or when FILE is -, read standard input.\n\n"
				"  -d\t\tdecode\n"
				"  -e\t\tencode\n"
				"  -m SIZE\tuse SIZE bytes of memory for buffers (suffixes: b, K, M)\n"
				"  -o FILE\twrite to FILE instead of standard output\n"
				"  -v\t\tverbose mode\n"
				"  -h\t\tdisplay this help and exit\n"
				"  -V\t\toutput version information and exit");
			return 0;
		case 'V':
			puts("b91enc " VERSION "\nCopyright (c) 2000-2005 Joachim Henke");
			return 0;
		default:
			fputs("Try `b91enc -h' for more information.\n", stderr);
			return 1;
		}

	if (obs < 0)
		obs = 65536;	/* memory usage defaults to 64 KiB */
	if (flg & FLG_D) {
		if (obs < 3) {
			fputs("SIZE must be >= 3 for decoding.\n", stderr);
			return 1;
		}
		ibs = ((obs - 1) << 3) / 15;
	} else {
		if (obs < 4) {
			fputs("SIZE must be >= 4 for encoding.\n", stderr);
			return 1;
		}
		ibs = ((obs - 2) << 4) / 29;
	}
	if (flg & FLG_VV)
		fprintf(stderr, "using %u bytes for buffers, input buffer: %u bytes\n", obs, ibs);
	if (!(obf = malloc(obs))) {
		fputs("\nCouldn't allocate buffer memory.\n", stderr);
		return 1;
	}

	if (optind < argc && strcmp(argv[optind], "-")) {
		ifn = argv[optind];
		if ((ifd = open(ifn, O_RDONLY)) == -1) {
			perror(ifn);
			return 1;
		}
	}
	if (ofn)
		if ((ofd = creat(ofn, S_IRUSR | S_IWUSR)) == -1) {
			perror(ofn);
			return 1;
		}

	b91_init();	/* initialise basE91 backend */
	if (flg & FLG_D) {
		ibf = &obf[1];	/* create overlapping buffers to use memory efficiently */
		if (flg & FLG_V) {
			fprintf(stderr, "decoding %s ...", ifn);
			decode();
			fputs("done\n", stderr);
		} else
			decode();
	} else {
		ibf = &obf[obs - ibs];	/* partial overlap */
		if (flg & FLG_V) {
			fprintf(stderr, "encoding %s ...", ifn);
			fprintf(stderr, "\t%.2f%%\n", encode() * 100);
		} else
			encode();
	}
	free(obf);

	return 0;
}
